<?php
/*
V4.92a 29 Aug 2006  (c) 2000-2006 John Lim. All rights reserved.
  Released under both BSD license and Lesser GPL library license. 
  Whenever there is any discrepancy between the two licenses, 
  the BSD license will take precedence.

  Latest version is available at http://adodb.sourceforge.net
  
  Portable version of oci8 driver, to make it more similar to other database drivers.
  The main differences are

   1. that the OCI_ASSOC names are in lowercase instead of uppercase.
   2. bind variables are mapped using ? instead of :<bindvar>

   Should some emulation of RecordCount() be implemented?
  
*/

// security - hide paths
if (!defined('ADODB_DIR')) die();

include_once(ADODB_DIR.'/drivers/adodb-oci8.inc.php');

class ADODB_oci8po extends ADODB_oci8 {
	var $databaseType = 'oci8po';
	var $dataProvider = 'oci8';
	var $metaColumnsSQL = "select lower(cname),coltype,width, SCALE, PRECISION, NULLS, DEFAULTVAL from col where tname='%s' order by colno"; //changed by smondino@users.sourceforge. net
	var $metaTablesSQL = "select lower(table_name),table_type from cat where table_type in ('TABLE','VIEW')";
	
	function ADODB_oci8po()
	{
		$this->_hasOCIFetchStatement = ADODB_PHPVER >= 0x4200;
		# oci8po does not support adodb extension: adodb_movenext()
//Mod:Start
        $this->NLS_DATE_FORMAT = 'RRRR-MM-DD HH24:MI:SS';
//Mod:End
	}
	
	function Param($name)
	{
		return '?';
	}
	
	function Prepare($sql,$cursor=false)
	{
		$sqlarr = explode('?',$sql);
		$sql = $sqlarr[0];
		for ($i = 1, $max = sizeof($sqlarr); $i < $max; $i++) {
			$sql .=  ':'.($i-1) . $sqlarr[$i];
		} 
//Mod:Start
        // \J̕ϊ
	    $head = strtolower(substr($sql, 0, 6));
	    if (in_array($head, array('insert', 'select', 'update', 'delete')))
	    {
	        $sql = preg_replace('/\bsize\b/i', '"SIZE"', $sql);
	        $sql = preg_replace('/\bstart\b/i', '"START"', $sql);
	    }
//Mod:End
		return ADODB_oci8::Prepare($sql,$cursor);
	}
	
	// emulate handling of parameters ? ?, replacing with :bind0 :bind1
	function _query($sql,$inputarr)
	{
		if (is_array($inputarr)) {
			$i = 0;
			if (is_array($sql)) {
				foreach($inputarr as $v) {
					$arr['bind'.$i++] = $v;
				} 
			} else {
				$sqlarr = explode('?',$sql);
				$sql = $sqlarr[0];
				foreach($inputarr as $k => $v) {
					$sql .=  ":$k" . $sqlarr[++$i];
				}
			}
		}
		return ADODB_oci8::_query($sql,$inputarr);
	}
//Mod:Start
    //--------------------------------------------------------------------------
    //--    ֐F Execute                                                  --
    //--    MethodF Execute()                                                --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * PosgreSQLłňꕔOracleŎgpłȂOgp߂̑ΉB
     * {͕sv
     */
	function &Execute($sql,$inputarr=false) 
	{
        // \J̕ϊ
	    $head = strtolower(substr($sql, 0, 6));
	    if (in_array($head, array('insert', 'select', 'update', 'delete')))
	    {
	        $sql = preg_replace('/\bsize\b/i', '"SIZE"', $sql);
	        $sql = preg_replace('/\bstart\b/i', '"START"', $sql);
	    }

	    return parent::Execute($sql, $inputarr);
	}

	
	/**
	* Usage:
	* Store BLOBs and CLOBs
	*
	* Example: to store $var in a blob
	*
	*	$conn->Execute('insert into TABLE (id,ablob) values(12,empty_blob())');
	*	$conn->UpdateBlob('TABLE', 'ablob', $varHoldingBlob, 'ID=12', 'BLOB');
	*	
	*	$blobtype supports 'BLOB' and 'CLOB', but you need to change to 'empty_clob()'.
	*
	*  to get length of LOB:
	*  	select DBMS_LOB.GETLENGTH(ablob) from TABLE
	*
	* If you are using CURSOR_SHARING = force, it appears this will case a segfault
	* under oracle 8.1.7.0. Run:
	*	 $db->Execute('ALTER SESSION SET CURSOR_SHARING=EXACT');
	* before UpdateBlob() then...
	*/
	function UpdateBlob($table,$column,$val,$where,$blobtype='BLOB')
	{
        if( strlen($val) > 0 ){
            return parent::UpdateBlob($table,$column,$val,$where,$blobtype);
        }

        $sql = "UPDATE $table set $column=NULL WHERE $where";
        $commit = $this->autoCommit;
        if ($commit) $this->BeginTrans();
        $rs = $this->_Execute($sql,$arr);
        $rez = !empty($rs);
        if($rs !== false) $rs->Close();
        if ($commit) $this->CommitTrans();
        return $rez;
    }

    //--------------------------------------------------------------------------
    //--    ֐F eʎ擾                                                 --
    //--    MethodF bm_GetTableUsage()                                       --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * e[u̗eʂ擾B
     * 
     * @param  array   $setting        [IN] :e[u̔z
     * @return array   e[uƂ̃TCY̔z
     */
    function bm_GetTableUsage($tables)
    {
        //[ `FbN ]
        if (!is_array($tables))
        {
            return false;
        }

        // eʎ擾
        $sql = "SELECT 
                    SEGMENT_NAME
                  , BYTES
                FROM
                    USER_SEGMENTS";
        $segments = $this->GetAssoc($sql);
        //[ G[ ]
        if (!is_array($segments))
        {
            return false;
        }

        $entities = array();
        //[ e[û̃TCY擾 ]
        foreach ($tables as $table)
        {
            $tableU = strtoupper($table);
            $tableL = strtolower($table);

            //[ e[ȗ݃`FbN ]
            if (!isset($segments[$tableU]))
            {
                return false;
            }

            $entities[$tableL] = $segments[$tableU];
        }

        //[ e[uɊ֘AZOg̗eʂǉ ]
        foreach ($segments as $name => $size)
        {
            // L[̃ZOgeʂǉ
            $ret = preg_match('/^(\w+)_PKEY$/', $name, $matches);
            $tableL = isset($matches[1]) ? strtolower($matches[1]) : null;
            if ($ret > 0 && isset($tableL) && isset($entities[$tableL]))
            {
                $entities[$tableL] += $size;
            }

            // LOB̃ZOgeʂǉ
            $ret = preg_match('/^(\w+)_\d+_LOB$/', $name, $matches);
            $tableL = isset($matches[1]) ? strtolower($matches[1]) : null;
            if ($ret > 0 && isset($tableL) && isset($entities[$tableL]))
            {
                $entities[$tableL] += $size;
            }

            // LOB index̃ZOgeʂǉ
            $ret = preg_match('/^(\w+)_\d+_LBI$/', $name, $matches);
            $tableL = isset($matches[1]) ? strtolower($matches[1]) : null;
            if ($ret > 0 && isset($tableL) && isset($entities[$tableL]))
            {
                $entities[$tableL] += $size;
            }
        }

        return $entities;
    }

    //--------------------------------------------------------------------------
    //--    ֐F tSQL擾                                      --
    //--    MethodF bm_GetDateTruncSql()                                     --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * tSQL̈ꕔ擾B
     * 
     * format̒lADODB_DATETYPE_YEARAADODB_DATETYPE_MONTHAADODB_DATETYPE_WEEKA
     * ADODB_DATETYPE_DAYAADODB_DATETYPE_HOURAADODB_DATETYPE_MINUTEAADODB_DATETYPE_SECOND̂ꂩ
     * 
     * @param  string  $colname        [IN] :J
     * @param  enum    $format         [IN] :P
     * @return string  tSQL̈ꕔ
     */
    function bm_GetDateTruncSql($colname, $format)
    {
        $format_map = array(ADODB_DATETYPE_YEAR   => 'YYYY'
                          , ADODB_DATETYPE_MONTH  => 'MM'
                          , ADODB_DATETYPE_WEEK   => 'D'
                          , ADODB_DATETYPE_DAY    => 'DD'
                          , ADODB_DATETYPE_HOUR   => 'HH24'
                          , ADODB_DATETYPE_MINUTE => 'MI');
        //[ T̏ꍇ ]
        if ($format == ADODB_DATETYPE_SECOND)
        {
            return $colname;
        }
        //[ ȊO ]
        if (isset($format_map[$format]))
        {
            return "trunc(".$colname.", '".$format_map[$format]."')";
        }

        //[ sȒl ]
        return "!Bad Argument(".$format.")!";
    }

    //--------------------------------------------------------------------------
    //--    ֐F 擾SQL擾                                  --
    //--    MethodF bm_GetDateLastDaySql()                                   --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * 擾SQL̈ꕔ擾B
     * 
     * @param  string  $colname        [IN] :J
     * @return string  擾SQL̈ꕔ
     */
    function bm_GetDateLastDaySql($colname)
    {
        return $this->bm_GetDateTruncSql("LAST_DAY(".$colname.")", ADODB_DATETYPE_DAY);
    }

    //--------------------------------------------------------------------------
    //--    ֐F YT̓t擾SQL擾                          --
    //--    MethodF bm_GetDayOfWeekSql()                                     --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * YT̓t擾SQL̈ꕔ擾B
     * 
     * week̒lADODB_DATETYPE_SUNDAYAADODB_DATETYPE_MONDAYAADODB_DATETYPE_TUESDAYA
     * ADODB_DATETYPE_WEDNESDAYAADODB_DATETYPE_THURSDAYAADODB_DATETYPE_SATURDAYAADODB_DATETYPE_SUNDAŶꂩ
     * 
     * @param  string  $colname        [IN] :J
     * @param  enum    $week           [IN] :j
     * @return string  擾SQL̈ꕔ
     */
    function bm_GetDayOfWeekSql($colname, $week)
    {
        $format_map = array(ADODB_DATETYPE_SUNDAY    => 0
                          , ADODB_DATETYPE_MONDAY    => 1
                          , ADODB_DATETYPE_TUESDAY   => 2
                          , ADODB_DATETYPE_WEDNESDAY => 3
                          , ADODB_DATETYPE_THURSDAY  => 4
                          , ADODB_DATETYPE_FRIDAY    => 5
                          , ADODB_DATETYPE_SATURDAY  => 6);
        //[ Ȓl ]
        if (isset($format_map[$week]))
        {
            return "trunc(".$colname.",'D') + ".$format_map[$week];
        }

        //[ sȒl ]
        return "!Bad Argument(".$week.")!";
    }

    //--------------------------------------------------------------------------
    //--    ֐F ԂID擾                                     --
    //--    MethodF bm_AutoIncrementID()                                     --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * ԂID擾B
     * V[PX́ue[u_J_SEQv(ꍇ́uJ_SEQv)ɂB
     * 
     * @param  string  $table       [IN] :e[u
     * @param  string  $column      [IN] :J
     * @return int     Ԕԍ
     */
    function bm_AutoIncrementID($table, $column)
    {
        // V[PX擾
        $seqname = strtoupper($table."_".$column."_SEQ");
        if (strlen($seqname) > 30)
        {
            $seqname =  strtoupper($column."_SEQ");
        }

        $sql = "SELECT ".$seqname.".currval FROM dual";
        return $this->getOne($sql);
    }

    //--------------------------------------------------------------------------
    //--    ֐F J̕ŏɂׂSQL擾                  --
    //--    MethodF bm_GetStringLengthSql()                                  --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2006-06-12 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
	/**
     * wJ̃f[^̕擾SQL擾B
     * 
     * @param  string  $colname        [IN] :J
     * @return string  tSQL̈ꕔ
     */
    function bm_GetStringLengthSql($colname)
    {
        return "length(".$colname.")";
    }
//Mod:End
}

/*--------------------------------------------------------------------------------------
		 Class Name: Recordset
--------------------------------------------------------------------------------------*/

class ADORecordset_oci8po extends ADORecordset_oci8 {

	var $databaseType = 'oci8po';
	
	function ADORecordset_oci8po($queryID,$mode=false)
	{
		$this->ADORecordset_oci8($queryID,$mode);
	}

	function Fields($colname)
	{
		if ($this->fetchMode & OCI_ASSOC) return $this->fields[$colname];
		
		if (!$this->bind) {
			$this->bind = array();
			for ($i=0; $i < $this->_numOfFields; $i++) {
				$o = $this->FetchField($i);
				$this->bind[strtoupper($o->name)] = $i;
			}
		}
		 return $this->fields[$this->bind[strtoupper($colname)]];
	}
	
	// lowercase field names...
	function &_FetchField($fieldOffset = -1)
	{
		 $fld = new ADOFieldObject;
 		 $fieldOffset += 1;
		 $fld->name = strtolower(OCIcolumnname($this->_queryID, $fieldOffset));
		 $fld->type = OCIcolumntype($this->_queryID, $fieldOffset);
		 $fld->max_length = OCIcolumnsize($this->_queryID, $fieldOffset);
		 if ($fld->type == 'NUMBER') {
		 	//$p = OCIColumnPrecision($this->_queryID, $fieldOffset);
			$sc = OCIColumnScale($this->_queryID, $fieldOffset);
			if ($sc == 0) $fld->type = 'INT';
		 }
		 return $fld;
	}
	/*
	function MoveNext()
	{
		if (@OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode)) {
			$this->_currentRow += 1;
			return true;
		}
		if (!$this->EOF) {
			$this->_currentRow += 1;
			$this->EOF = true;
		}
		return false;
	}*/

	// 10% speedup to move MoveNext to child class
	function MoveNext() 
	{
		if(@OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode)) {
		global $ADODB_ANSI_PADDING_OFF;
			$this->_currentRow++;
			
			if ($this->fetchMode & OCI_ASSOC) $this->_updatefields();
			if (!empty($ADODB_ANSI_PADDING_OFF)) {
				foreach($this->fields as $k => $v) {
					if (is_string($v)) $this->fields[$k] = rtrim($v);
				}
			}
			return true;
		}
		if (!$this->EOF) {
			$this->EOF = true;
			$this->_currentRow++;
		}
		return false;
	}	
	
	/* Optimize SelectLimit() by using OCIFetch() instead of OCIFetchInto() */
	function &GetArrayLimit($nrows,$offset=-1) 
	{
		if ($offset <= 0) {
			$arr = $this->GetArray($nrows);
			return $arr;
		}
		for ($i=1; $i < $offset; $i++) 
			if (!@OCIFetch($this->_queryID)) {
				$arr = array();
				return $arr;
			}
		if (!@OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode)) {
			$arr = array();
			return $arr;
		}
		if ($this->fetchMode & OCI_ASSOC) $this->_updatefields();
		$results = array();
		$cnt = 0;
		while (!$this->EOF && $nrows != $cnt) {
			$results[$cnt++] = $this->fields;
			$this->MoveNext();
		}
		
		return $results;
	}

	// Create associative array
	function _updatefields()
	{
		if (ADODB_ASSOC_CASE == 2) return; // native
	
		$arr = array();
		$lowercase = (ADODB_ASSOC_CASE == 0);
		
		foreach($this->fields as $k => $v) {
			if (is_integer($k)) $arr[$k] = $v;
			else {
				if ($lowercase)
					$arr[strtolower($k)] = $v;
				else
					$arr[strtoupper($k)] = $v;
			}
		}
		$this->fields = $arr;
	}
	
	function _fetch() 
	{
		$ret = @OCIfetchinto($this->_queryID,$this->fields,$this->fetchMode);
		if ($ret) {
		global $ADODB_ANSI_PADDING_OFF;
	
				if ($this->fetchMode & OCI_ASSOC) $this->_updatefields();
				if (!empty($ADODB_ANSI_PADDING_OFF)) {
					foreach($this->fields as $k => $v) {
						if (is_string($v)) $this->fields[$k] = rtrim($v);
					}
				}
		}
		return $ret;
	}
	
}


?>