var entry_editor_curr_onload = window.onload;

/**
 * 投稿の編集クラス
 * 
 */
function EntryEditor()
{
    /**
     * @var    object  article_edit_focus  編集対象のフォーム
     */
    this.article_edit_focus = null;

    /**
     * @var    string  image_path          画像のパス
     */
    this.image_path = 'images/';

    /**
     * @var    bool    edited              編集済みかどうか
     */
    this.edited     = false;

    /**
     * プレビューの内容を更新する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.refresh_preview = function(target_form)
    {
        if (typeof(target_form) == 'undefined')
        {
            return;
        }

        NoChangeAlert_setConfirm();

        var target_id = target_form.name + '_preview';
        var text      = target_form.value;

        fixed_text = text.replace(/\r\n/g,"\n");
        fixed_text = fixed_text.replace(/\r/g  ,"\n");

        // 行数の調整
        var n_line = fixed_text.split("\n").length;

        if (n_line > 20)
        {
            n_line = 20;
        }
        target_form.rows = n_line + 5;

        // プレビューを書く
        if (document.getElementById('i_newline_enable') && document.getElementById('i_newline_enable').checked)
        {
            text = fixed_text.replace(/\n/g,"<br>");
        }

        text = dnblogtag_to_htmltag(text, this.image_path, false);
        text = text.replace(/\[\{keyword\|(.*?)\}\]/img, '$1');
        document.getElementById(target_id).innerHTML = text+'<img src="'+this.image_path+'sp.gif" width="1" height="1">';
    }

    /**
     * 太字タグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_bold = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<strong>', '</strong>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * 斜体タグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_italic = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<em>', '</em>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * 下線タグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_underline = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<u>', '</u>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * 左寄せタグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_left = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<div align="left">', '</div>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * 中寄せタグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_center = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<div align="center">', '</div>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * 右寄せタグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_right = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<div align="right">', '</div>');
        this.refresh_preview(this.article_edit_focus);
    }
   
    /**
     * アンカータグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_anchor = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        text_url = window.prompt('URLを入力してください。','http://');
        if (text_url != null)
        {
            this._wrap('<a href="' + text_url + '" target="_blank">', '</a>');
        }
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * キーワードタグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.insert_keyword = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        text_keyword = window.prompt('キーワードを入力してください。', this._get_selected());
        if (text_keyword != null)
        {
            this._replace(text_keyword);
            this._wrap('[{keyword|', '}]');
        }
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * 文字サイズタグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     * @param  string  size        [IN]:文字サイズ
     */
    this.insert_fontsize = function(target_form, size)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<font size="' + size + '">', '</font>');
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * 文字色タグを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     * @param  string  color       [IN]:文字色
     */
    this.insert_fontcolor = function(target_form, color)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._wrap('<font color="' + color + '">', '</font>');
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * 絵文字を挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     * @param  string  image_uri       [IN]:絵文字URI
     */
    this.insert_pictgraph = function(target_form, image_name)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;
        this._replace('[{icon|' + image_name + '}]');
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * ファイルを挿入する。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     * @param  string  file_id     [IN]:ファイルID
     * @param  string  alignment   [IN]:回り込み
     * @param  string  width       [IN]:幅
     * @param  string  height      [IN]:高さ
     * @param  string  alter_text  [IN]:代替テキスト
     * @param  bool    is_anchor   [IN]:リンクにするかどうか
     */
    this.insert_file = function(file_id, alignment, width, height, alter_text, is_anchor)
    {
        entry_editor_close_all();

        // 回り込みなし
        if (alignment == 'none')
        {
            alignment = '';
        }

        if (is_anchor)
        {
            // 画像タグ
            var pretext  = '[{file|'+file_id+'|'+width+'|'+height+'|'+alignment+'|'+alter_text+'}]';
            var posttext = '';
        }
        else
        {
            // 画像タグ
            var pretext  = '[{image|'+file_id+'|'+width+'|'+height+'|'+alignment+'|'+alter_text+'}]';
            var posttext = '';
        }

        this._wrap(pretext, posttext);
        this.refresh_preview(this.article_edit_focus);
    }

    /**
     * 文字サイズ選択レイヤーの開閉を切り替える。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.open_close_fontsize = function(target_form)
    {
        var target_id    = 'layer_fontsize_' + target_form.name;
        var target_style = document.getElementById(target_id).style;

        // 閉から開
        if (target_style.display == 'none')
        {
            entry_editor_close_all();
            target_style.display = 'block';
        }
        // 開から閉
        else
        {
            entry_editor_close_all();
        }
    }

    /**
     * 文字色選択レイヤーの開閉を切り替える。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.open_close_fontcolor = function(target_form)
    {
        var target_id    = 'layer_fontcolor_' + target_form.name;
        var target_style = document.getElementById(target_id).style;

        // 閉から開
        if (target_style.display == 'none')
        {
            entry_editor_close_all();
            target_style.display = 'block';
        }
        // 開から閉
        else
        {
            entry_editor_close_all();
        }
    }

    /**
     * 絵文字選択レイヤーの開閉を切り替える。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.open_close_pictgraph = function(target_form)
    {
        var target_id    = 'layer_pictgraph_' + target_form.name;
        var target_style = document.getElementById(target_id).style;

        // 閉から開
        if (target_style.display == 'none')
        {
            entry_editor_close_all();
            target_style.display = 'block';
        }
        // 開から閉
        else
        {
            entry_editor_close_all();
        }
    }

    /**
     * ファイル選択画面を開く。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.open_file = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;

        file_selecter_uri = 'index.php?module=User&action=EntrySelFile&alignment=none';

        file_selecter_window = window.open(encodeURI(file_selecter_uri), 'EntrySelFile'
                , 'width=800,height=600,scrollbars=yes,location=no,menubar=no');
        // ファイルが開けた
    }

    /**
     * 管理者用ファイル選択画面を開く。
     * 
     * @param  object  target_form [IN]:更新対象フォーム
     */
    this.open_file_admin = function(target_form)
    {
        entry_editor_close_all();

        target_form.focus();
        this.article_edit_focus = target_form;

        file_selecter_uri = 'index.php?module=Admin&action=AEntrySelFile';
        // 回り込み初期値
        file_selecter_uri = file_selecter_uri + '&alignment=none';
        // 代替テキスト
        file_selecter_uri = file_selecter_uri + '&alter_text=' + this._get_selected();
        // ブログシステムID(管理者用)
        file_selecter_uri = file_selecter_uri + '&i_system_id=' + document.frm.i_system_id.value;

        file_selecter_window = window.open(encodeURI(file_selecter_uri), 'EntrySelFile'
                , 'width=800,height=600,scrollbars=yes,location=no,menubar=no');
        // ファイルが開けた
    }

    /**
     * 選択部分を取得する。
     * 
     * @return string  選択された文字列
     */
    this._get_selected = function()
    {
        var t_form        = this.article_edit_focus;
        var selected_text = '';
   
        // Firefox,Opera
        if (typeof(t_form.selectionStart) != 'undefined' && 
            typeof(t_form.selectionEnd  ) != 'undefined')
        {
            var start     = t_form.selectionStart;
            var end       = t_form.selectionEnd;
            selected_text = t_form.value.substring(start, end);
        }
        // IE
        else if (document.selection && document.selection.createRange())
        {
            var text_range = document.selection.createRange();
            selected_text  = text_range.text;
        }
   
        return selected_text;
    }

    /**
     * 選択部分を文字列で置き換える
     * 
     * @param  string  reptext     [IN]:置き換える文字列
     */
    this._replace = function(reptext)
    {
        var t_form = this.article_edit_focus;

        // Firefox,Opera
        if (typeof(t_form.selectionStart) != 'undefined' && 
            typeof(t_form.selectionEnd  ) != 'undefined')
        {
            var length = t_form.textLength;
            var start  = t_form.selectionStart;
            var end    = t_form.selectionEnd;

            // テキスト編集
            var fixed_text = t_form.value.substring(0, start);
            fixed_text = fixed_text + reptext;
            fixed_text = fixed_text + t_form.value.substring(end, length);
            t_form.value = fixed_text;

            // 選択範囲設定
            t_form.selectionStart = start;
            t_form.selectionEnd   = start + reptext.length;
        }
        // IE
        else if (document.selection && document.selection.createRange())
        {
            // テキスト編集
            var t_range_text  = document.selection.createRange();
            var t_range_start = t_range_text.duplicate();
            var wrap_text     = t_range_text.text;
            t_range_text.text = reptext;

           // 選択範囲設定
            t_range_start.setEndPoint('StartToStart', t_range_text);
            t_range_start.select();
        }
        // その他
        else
        {
            t_form.value += reptext;
        }
    }

    /**
     * 選択部分を開始文字列と終了文字列で囲む
     * 
     * @param  string  pretext     [IN]:開始文字列
     * @param  string  posttext    [IN]:終了文字列
     */
    this._wrap = function(pretext, posttext)
    {
        var t_form = this.article_edit_focus;

        // Firefox,Opera
        if (typeof(t_form.selectionStart) != 'undefined' && 
            typeof(t_form.selectionEnd  ) != 'undefined')
        {
            var length = t_form.textLength;
            var start  = t_form.selectionStart;
            var end    = t_form.selectionEnd;

            // テキスト編集
            var fixed_text = t_form.value.substring(0, start);
            fixed_text = fixed_text + pretext;
            fixed_text = fixed_text + t_form.value.substring(start, end);
            fixed_text = fixed_text + posttext;
            fixed_text = fixed_text + t_form.value.substring(end, length);
            t_form.value = fixed_text;

            // 選択範囲設定
            t_form.selectionStart = start + pretext.length;
            t_form.selectionEnd   = end   + pretext.length;
        }
        // IE
        else if (document.selection && document.selection.createRange())
        {
            // 開始タグ終了地点まで
            var t_range_start  = document.selection.createRange();
            var wrap_text      = t_range_start.text;
            t_range_start.text = pretext;
            // テキスト終了地点まで
            var t_range_text   = t_range_start.duplicate();
            t_range_text.text  = wrap_text;
            // 終了タグ終了地点まで
            var t_range_end    = t_range_text.duplicate();
            t_range_end.text   = posttext;
            // 開始タグ終了地～テキスト終了地点を選択
            t_range_start.setEndPoint('StartToStart', t_range_text);
            t_range_start.select();
        }
        // その他
        else
        {
            t_form.value += pretext +  posttext;
        }
    }
}

/**
 * @var    object  entry_editor    投稿編集クラス
 */
var entry_editor = new EntryEditor;

function entry_editor_init()
{
    // 他のonloadイベント実行
    if (entry_editor_curr_onload)
    {
        entry_editor_curr_onload();
    }

    refresh_all_preview();

    // 最初の画面遷移の場合のみ警告しない
    if (! entry_editor.edited)
    {
        NoChangeAlert_unsetConfirm();
    }
}

function entry_editor_close_all()
{
    document.getElementById('layer_fontsize_i_entry_text_more').style.display = 'none';
    document.getElementById('layer_fontsize_i_entry_text').style.display = 'none';
    document.getElementById('layer_fontcolor_i_entry_text_more').style.display = 'none';
    document.getElementById('layer_fontcolor_i_entry_text').style.display = 'none';
    document.getElementById('layer_pictgraph_i_entry_text_more').style.display = 'none';
    document.getElementById('layer_pictgraph_i_entry_text').style.display = 'none';
}

function insert_file(file_id, align, width, height, alt, is_anchor)
{
    // ファイル画像タグを挿入
    entry_editor.insert_file(file_id, align, width, height, alt, is_anchor);
}

/**
 * 全てのプレビューの内容を更新する。
 * 
 */
function refresh_all_preview()
{
    entry_editor.refresh_preview(document.frm.i_entry_text);
    entry_editor.refresh_preview(document.frm.i_entry_text_more);
}

window.onload = entry_editor_init;
setTimeout('entry_editor_init();', 500);
